import numpy as np


class Kmeans:
    def __init__(self, max_iterations=100, tolerance=1e-6) -> None:
        self.max_iterations = max_iterations
        self.tolerance = tolerance

    def kmeans(self, X, k):
        centroids = X[np.random.choice(X.shape[0], k, replace=False), :]
        for i in range(self.max_iterations):
            distances = np.array(
                [self._distance(X, centroid) for centroid in centroids]
            )
            clusters = np.argmin(distances, axis=0)
            new_centroids = np.array([X[clusters == j].mean(axis=0) for j in range(k)])
            if np.linalg.norm(new_centroids - centroids) < self.tolerance:
                break
            centroids = new_centroids

        return centroids, clusters

    def _distance(self, X, centroid):
        return np.linalg.norm(X - centroid, axis=1)



import cv2
from MDC.utilities.kmeans import Kmeans
import numpy as np
import matplotlib.pyplot as plt
import os
import colorsys


def image(args):
    _get_centroids(args)


def _get_centroids(args):
    img_path = args.image_name[0]
    if not os.path.isfile(img_path):
        raise Exception("Incorrect file path")
    if args.t:
        img_title = args.t
    else:
        img_basename = os.path.basename(img_path)
        parts = []
        for p in img_basename.split("."):
            parts.append(p)
        img_title = parts[0]

    img = cv2.imread(img_path)
    img = cv2.resize(img, (600, 400))
    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    img = img.reshape((img.shape[0] * img.shape[1], 3))
    kmeans = Kmeans()

    centroids, clusters = kmeans.kmeans(img, k=args.c)
    _get_palette(centroids, img_title, args)


def _get_palette(centroids, img_title, args):
    start = 0
    width = int(args.c * 50)
    height = int(width / args.c)
    pallete = np.zeros((height, width, 3), np.uint8)
    font = cv2.FONT_HERSHEY_PLAIN
    font_scale = 0.5
    for centroid in centroids:
        r, g, b = int(centroid[0]), int(centroid[1]), int(centroid[2])
        end = start + height
        cv2.rectangle(pallete, (start, 0), (int(end), height), (r, g, b), -1)
        if args.hex:
            h, s, v = colorsys.rgb_to_hsv(r / 255.0, g / 255.0, b / 255.0)
            rgb_color = colorsys.hsv_to_rgb(h, s, v)
            hex_color = "#{:02x}{:02x}{:02x}".format(
                int(rgb_color[0] * 255),
                int(rgb_color[1] * 255),
                int(rgb_color[2] * 255),
            )
            text = hex_color.upper()
            text_size = cv2.getTextSize(text, font, font_scale, 1)[0]
            text_x = start + int((height - text_size[0]) / 2)
            text_y = int(height * 0.9)
            cv2.putText(
                pallete, text, (text_x, text_y), font, font_scale, (255, 255, 255), 1
            )
        start = end
    plt.figure()
    plt.axis("off")
    plt.title(img_title, fontdict={"fontweight": "bold"})
    plt.imshow(pallete)
    plt.savefig(img_title)
    plt.show()



from argparse import ArgumentParser, ArgumentTypeError
import sys
from typing import NamedTuple, List, Tuple, Dict, Any
import MDC.functions

Argument = Tuple[List[str], Dict[str, Any]]


class Command(NamedTuple):
    name: str
    description: str
    arguments: List[Argument]


def time(time: str):
    parts = []
    for p in time.split(":"):
        parts.append(int(p))
    hours, minutes = parts[0], parts[1]
    seconds = parts[2] if len(parts) > 2 else 0

    if hours < 0 or not (0 <= minutes <= 59) or not (0 <= seconds <= 59):
        raise ArgumentTypeError

    return hours * 3600 + minutes * 60 + seconds


commands = [
    Command(
        name="video",
        description="Convert video to a palette",
        arguments=[
            (
                ["video_name"],
                {
                    "help": "The path of the video you want to convert.",
                    "type": str,
                    "nargs": "+",
                },
            ),
            (
                ["-s"],
                {
                    "help": "Convert video from this time. (hh:mm:ss)",
                    "type": time,
                    "default": None,
                },
            ),
            (
                ["-e"],
                {
                    "help": "Convert video up to this time. (hh:mm:ss)",
                    "type": time,
                    "default": None,
                },
            ),
            (
                ["-t"],
                {
                    "help": "Set a palette title.",
                    "type": str,
                    "default": None,
                },
            ),
        ],
    ),
    Command(
        name="image",
        description="Convert image to a palette of colors",
        arguments=[
            (
                ["image_name"],
                {
                    "help": "The path of the image you want to convert.",
                    "type": str,
                    "nargs": "+",
                },
            ),
            (
                ["-c"],
                {
                    "help": "Number of colors in the palette.",
                    "type": int,
                    "default": 5,
                },
            ),
            (
                ["-t"],
                {
                    "help": "Set a chart title.",
                    "type": str,
                    "default": None,
                },
            ),
            (
                ["-hex"],
                {
                    "help": "Shows the hex color code of each color on the palette.",
                    "type": bool,
                    "nargs": "?",
                    "const": True,
                    "default": False,
                },
            ),
        ],
    ),
]


def get_parser():
    parser = ArgumentParser(prog="mdc")
    subparsers = parser.add_subparsers(title="commands")

    for command in commands:
        sub = subparsers.add_parser(name=command.name, description=command.description)
        sub.set_defaults(func=MDC.functions.__dict__.get(command.name))
        for args, kwargs in command.arguments:
            sub.add_argument(*args, **kwargs)
    return parser


def main():
    parser = get_parser()
    args = parser.parse_args()
    if "func" not in args:
        parser.print_help()
    args.func(args)



import cv2
from MDC.utilities.kmeans import Kmeans
import numpy as np
import matplotlib.pyplot as plt
import os


def video(args):
    _get_centroids(args)


def _get_centroids(args):
    kmeans = Kmeans()
    video_path = args.video_name[0]

    if not os.path.isfile(video_path):
        raise Exception("Incorrect file path")

    if args.t:
        video_title = args.t
    else:
        video_basename = os.path.basename(video_path)
        parts = []
        for p in video_basename.split("."):
            parts.append(p)
        video_title = parts[0]

    video = cv2.VideoCapture(video_path)
    fps = video.get(cv2.CAP_PROP_FPS)
    start_frame, end_frame = 0, video.get(cv2.CAP_PROP_FRAME_COUNT)
    if args.s or args.e:
        if args.s:
            start_frame = int(fps * args.s)
        if args.e:
            end_frame = int(fps * args.e)
    if end_frame <= start_frame:
        raise Exception("The end time must be greater than the start time.")

    counter, frames, status, centroids = 0, 0, True, []
    while status:
        status, img = video.read()

        if frames >= start_frame and frames <= end_frame:
            if counter == 10 and status == 1:
                img = cv2.resize(img, (256, 144))
                img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                img = img.reshape((img.shape[0] * img.shape[1], 3))
                centroid, clusters = kmeans.kmeans(img, 1)
                centroids.append(centroid)
                counter = 0
            counter += 1
        elif frames >= end_frame:
            break
        frames += 1
    centroids = np.array(centroids)
    _get_palette(centroids, video_title)


def _get_palette(centroids, video_title):
    start = 0
    width = centroids.shape[0] * 50
    height = int(width * (9 / 20))
    pallete = np.zeros((height, width, 3), np.uint8)
    for centroid in centroids:
        end = start + 50
        r, g, b = int(centroid[0][0]), int(centroid[0][1]), int(centroid[0][2])
        cv2.rectangle(pallete, (start, 0), (int(end), height), (r, g, b), -1)
        start = end
    plt.figure()
    plt.axis("off")
    plt.title(video_title, fontdict={"fontweight": "bold"})
    plt.imshow(pallete)
    plt.savefig(video_title)
    plt.show()
